@@ -131,11 +131,60 @@ def prompt():
 			colorize("> ", "94m")
 		return input_(msg)
 
+def dump_vmcode(farth_obj, filename):
+	s = farth_obj.vm.gen_string()
+	f = open(filename, "w")
+	f.write(s)
+	f.close()
+
+def invert_dict(d):
+	newd = {}
+	for i in d:
+		newd[d[i]] = i
+	
+	return newd
+
+import re
+
+def encode(code, bytes_):
+	newcode = ''
+	
+	for i in code.split("\n"):
+		j = re.findall(r'(".*?"|[^ ]+)', i)
+		args = "\x98" + "\x98".join(map(str, j[1:])) if len(j) > 1 else ''
+		newcode += bytes_[j[0].lower()] + args
+		newcode += "\x99"
+	
+	return newcode[0:-1]
+
+def decode(code, bytes_):
+	newcode = ''
+	
+	for i in code.split("\x99"):
+		j = re.findall(r'(".*?"|[^\x98]+)', i)
+		args = " " + " ".join(map(str, j[1:])) if len(j) > 1 else ''
+		newcode += bytes_[j[0]] + args
+		newcode += "\n"
+	
+	return newcode[0:-1]
+
+def compile_to_bytecode(code, bytes_, filename):
+	bytecode = encode(code, bytes_)
+	f = open(filename, "wb")
+	f.write(b'FARTHBIN\n')
+	f.write(bytes(bytecode, "utf8"))
+	f.close()
+
 if __name__ == "__main__":
 	f = farth.Farth()
 	arg_parser = argparse.ArgumentParser(
 		description="Some sort of Forth implementation")
 	arg_parser.add_argument("filename", nargs="*", help="File to be executed")
+	arg_parser.add_argument("-d", "--dump", action="store_true",
+		help="Dump VM code into file")
+	arg_parser.add_argument("-b", "--bytecode", action="store_true",
+		help="Translate code to VM bytecode")
+	arg_parser.add_argument("-o", "--output", nargs=1, help="Output file")
 	args = arg_parser.parse_args()
 	
 	if args.filename:
@@ -147,7 +196,20 @@ if __name__ == "__main__":
 			print("File '%s' doesn't exist" %args.filename[0])
 			sys.exit(1)
 		
-		f.execute_string(code)
+		if args.dump and args.output:
+			f.compile_string(code)
+			dump_vmcode(f, args.output[0])
+		elif args.bytecode and args.output:
+			f.compile_string(code)
+			compile_to_bytecode(f.vm.gen_string(), invert_dict(f.vm.bytes),
+				args.output[0])
+		else:
+			if code.startswith("FARTHBIN\n"):
+				code = decode(code[9:], f.vm.bytes)
+				f.vm = farth.FarthVM(code, f)
+				f.vm.execute()
+			else:
+				f.compile_and_execute(code)
 		sys.exit(0)
 	
 	if os.path.exists(histpath):
@@ -172,24 +234,19 @@ Some sort of Forth implementation"""[1:]
 		else:
 			print(msg)
 	
-	i_before = f.i
-	
 	while True:
 		try:
 			readline.set_completer(Completer(f.words).complete)
 			s = prompt()
 			if s:
-				f.execute_string(s)
-			elif s == ".quit":
-				sys.exit(0)
+				f.compile_and_execute(s)
+			if f.vm.pc is None:
+				raise EOFError
 		except EOFError:
 			a = input_("\nAre You sure You want to exit? [Y/n]").lower()
 			if len(a) == 0 or a == "y":
 				sys.exit(0)
 		except farth.FarthError as error:
 			print(error)
-			f.i += f.i - i_before - 1
 		except KeyboardInterrupt:
 			print('')
-		else:
-			i_before = f.i
